@@ -8,51 +8,47 @@
 # License, version 2.1 as published by the Free Software Foundation.
 # See the file "COPYING" for the exact licensing terms.
 
-require 'dbus/type'
-require 'dbus/introspect'
-require 'dbus/error'
-require 'dbus/export'
-require 'dbus/bus.rb'
-require 'dbus/marshall'
-require 'dbus/message'
-require 'dbus/matchrule'
-require 'dbus/auth'
+require_relative "dbus/api_options"
+require_relative "dbus/auth"
+require_relative "dbus/bus"
+require_relative "dbus/error"
+require_relative "dbus/export"
+require_relative "dbus/introspect"
+require_relative "dbus/logger"
+require_relative "dbus/marshall"
+require_relative "dbus/matchrule"
+require_relative "dbus/message"
+require_relative "dbus/message_queue"
+require_relative "dbus/proxy_object"
+require_relative "dbus/proxy_object_factory"
+require_relative "dbus/proxy_object_interface"
+require_relative "dbus/type"
+require_relative "dbus/xml"
 
-require 'socket'
-require 'thread'
-
-unless 0.respond_to?(:ord)
-  # Backward compatibility with Ruby 1.8.6, see http://www.pubbs.net/ruby/200907/65871/
-  class Integer
-    def ord; self; end
-  end
-end
+require "socket"
+require "thread"
 
 # = D-Bus main module
 #
 # Module containing all the D-Bus modules and classes.
 module DBus
   # Default socket name for the system bus.
-  SystemSocketName = "unix:path=/var/run/dbus/system_bus_socket"
+  SystemSocketName = "unix:path=/var/run/dbus/system_bus_socket".freeze
 
   # Byte signifying big endianness.
-  BIG_END = ?B
+  BIG_END = "B".freeze
   # Byte signifying little endianness.
-  LIL_END = ?l
+  LIL_END = "l".freeze
 
   # Byte signifying the host's endianness.
   HOST_END = if [0x01020304].pack("L").unpack("V")[0] == 0x01020304
-    LIL_END
-  else
-    BIG_END
-  end
+               LIL_END
+             else
+               BIG_END
+             end
 
   # General exceptions.
 
-  # Exception raised when an invalid packet is encountered.
-  class InvalidPacketException < Exception
-  end
-
   # Exception raised when there is a problem with a type (may be unknown or
   # mismatch).
   class TypeException < Exception
@@ -63,16 +59,8 @@ module DBus
   class IncompleteBufferException < Exception
   end
 
-  # Exception raised when a method has not been implemented (yet).
-  class MethodNotImplemented < Exception
-  end
-
-  # Exception raised when a method is invoked with invalid
-  # parameters (wrong number or type).
-  class InvalidParameters < Exception
-  end
-
   # Exception raised when an invalid method name is used.
+  # FIXME: use NameError
   class InvalidMethodName < Exception
   end
 
