@@ -8,7 +8,8 @@
 # License, version 2.1 as published by the Free Software Foundation.
 # See the file "COPYING" for the exact licensing terms.
 
-require 'thread'
+require "thread"
+require "dbus/core_ext/class/attribute"
 
 module DBus
   # Exported object type
@@ -19,14 +20,14 @@ module DBus
   class Object
     # The path of the object.
     attr_reader :path
-    # The interfaces that the object supports.
-    # intfs: Hash: String => Interface
-    # @@intfs_hash simulates a class attribute by being a hash Class => intfs
-    @@intfs_hash = {DBus::Object => nil}
+    # The interfaces that the object supports. Hash: String => Interface
+    my_class_attribute :intfs
+    self.intfs = {}
+
     # The service that the object is exported by.
     attr_writer :service
 
-    @@cur_intf = nil            # Interface
+    @@cur_intf = nil # Interface
     @@intfs_mutex = Mutex.new
 
     # Create a new object with a given _path_.
@@ -36,59 +37,34 @@ module DBus
       @service = nil
     end
 
-    def self.intfs
-      if self.equal? DBus::Object
-        @@intfs_hash[DBus::Object]
-      else
-        @@intfs_hash[self] || self.superclass.intfs
-      end
-    end
-
-    def self.intfs= param
-      @@intfs_hash[self] = param
-    end
-
-    def intfs
-      self.class.intfs
-    end
-
-    def intfs= param
-      self.class.intfs = param
-    end
-
-    # State that the object implements the given _intf_.
-    def implements(intf)
-      # use a setter
-      self.intfs = (self.intfs || {}).merge({intf.name => intf})
-    end
-
     # Dispatch a message _msg_ to call exported methods
     def dispatch(msg)
       case msg.message_type
       when Message::METHOD_CALL
         reply = nil
         begin
-          if not self.intfs[msg.interface]
+          if !intfs[msg.interface]
             raise DBus.error("org.freedesktop.DBus.Error.UnknownMethod"),
-            "Interface \"#{msg.interface}\" of object \"#{msg.path}\" doesn't exist"
+                  "Interface \"#{msg.interface}\" of object \"#{msg.path}\" doesn't exist"
           end
-          meth = self.intfs[msg.interface].methods[msg.member.to_sym]
-          if not meth
+          meth = intfs[msg.interface].methods[msg.member.to_sym]
+          if !meth
             raise DBus.error("org.freedesktop.DBus.Error.UnknownMethod"),
-            "Method \"#{msg.member}\" on interface \"#{msg.interface}\" of object \"#{msg.path}\" doesn't exist"
+                  "Method \"#{msg.member}\" on interface \"#{msg.interface}\" of object \"#{msg.path}\" doesn't exist"
           end
           methname = Object.make_method_name(msg.interface, msg.member)
           retdata = method(methname).call(*msg.params)
-          retdata =  [*retdata]
+          retdata = [*retdata]
 
           reply = Message.method_return(msg)
           meth.rets.zip(retdata).each do |rsig, rdata|
             reply.add_param(rsig.type, rdata)
           end
         rescue => ex
-          reply = ErrorMessage.from_exception(ex).reply_to(msg)
+          dbus_msg_exc = msg.annotate_exception(ex)
+          reply = ErrorMessage.from_exception(dbus_msg_exc).reply_to(msg)
         end
-        @service.bus.send(reply.marshall)
+        @service.bus.message_queue.push(reply)
       end
     end
 
@@ -96,9 +72,15 @@ module DBus
     # belong to.
     def self.dbus_interface(s)
       @@intfs_mutex.synchronize do
-        unless @@cur_intf = (self.intfs && self.intfs[s])
+        @@cur_intf = intfs[s]
+        if !@@cur_intf
           @@cur_intf = Interface.new(s)
-          self.intfs = (self.intfs || {}).merge({s => @@cur_intf})
+          # As this is a mutable class_attr, we cannot use
+          #   self.intfs[s] = @@cur_intf                      # Hash#[]=
+          # as that would modify parent class attr in place.
+          # Using the setter lets a subclass have the new value
+          # while the superclass keeps the old one.
+          self.intfs = intfs.merge(s => @@cur_intf)
         end
         yield
         @@cur_intf = nil
@@ -117,7 +99,7 @@ module DBus
     def self.dbus_method(sym, protoype = "", &block)
       raise UndefinedInterface, sym if @@cur_intf.nil?
       @@cur_intf.define(Method.new(sym.to_s).from_prototype(protoype))
-      define_method(Object.make_method_name(@@cur_intf.name, sym.to_s), &block) 
+      define_method(Object.make_method_name(@@cur_intf.name, sym.to_s), &block)
     end
 
     # Emits a signal from the object with the given _interface_, signal
@@ -138,10 +120,10 @@ module DBus
     end
 
     ####################################################################
-    private
 
     # Helper method that returns a method name generated from the interface
     # name _intfname_ and method name _methname_.
+    # @api private
     def self.make_method_name(intfname, methname)
       "#{intfname}%%#{methname}"
     end
